Index: apps/app_enumlookup.c
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/app_enumlookup.c,v
retrieving revision 1.14
diff -u -r1.14 app_enumlookup.c
--- apps/app_enumlookup.c	21 Apr 2005 06:02:43 -0000	1.14
+++ apps/app_enumlookup.c	2 May 2005 22:56:44 -0000
@@ -1,14 +1,35 @@
 /*
  * Asterisk -- A telephony toolkit for Linux.
  *
- * Time of day - Report the time of day
+ * ENUM Lookup - Old Fashioned and Next Generation
  * 
  * Copyright (C) 1999 - 2005, Digium, Inc.
  *
  * Mark Spencer <markster@digium.com>
  *
+ * EnumLookupNG added by Edwin Groothuis <edwin@mavetju.org>
+ *
  * This program is free software, distributed under the terms of
  * the GNU General Public License
+ *
+ * Example macro for EnumLookupNG:
+ * exten => s,1,NoOp(ENUM TEST)
+ * exten => s,n,EnumLookupNG(16503816199,10,20,30,40)
+ * exten => s,n,NoOp(NOT SUPPORTED)
+ * exten => s,n,GoTo(2)
+ * exten => s,10,NoOp(VOIP)
+ * exten => s,n,NoOp(${ENUM})
+ * exten => s,n,GoTo(2)
+ * exten => s,20,NoOp(TEL)
+ * exten => s,n,NoOp(${ENUM})
+ * exten => s,n,GoTo(2)
+ * exten => s,30,NoOp(MAILTO)
+ * exten => s,n,AGI(record-and-mail.pl|${ENUM})
+ * exten => s,n,Hangup()
+ * exten => s,40,NoOp(FAIL)
+ * exten => s,n,NoOp(${ENUM})
+ * exten => s,n,Hangup()
+ *
  */
 
 #include "asterisk/lock.h"
@@ -27,13 +48,25 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-static char *tdesc = "ENUM Lookup";
+static char *tdesc = "ENUM Lookup apps";
+
+static char *new_app = "EnumLookupNG";
+
+static char *new_synopsis = "Lookup number in ENUM - Next Generation Implementation";
+
+static char *new_descrip = 
+"EnumLookup(exten,#VOIP,#TEL,#MAILTO,#FAIL):  Looks up an extension via ENUM, sets the variable 'ENUM' and jumps to the specified priority.\n"
+"For VoIP URIs this variable will look like 'TECHNOLOGY/URI' with the appropriate technology.\n"
+"Returns -1 on hangup, or 0 on completion regardless of whether the lookup was successful. \n"
+"Currently, the enumservices TEL, SIP, H323, IAX, IAX2 and MAILTO are recognized. \n"
+"For example: EnumLookup(12345,30,40,50,60) will lookup the extensions 12345. If it is a VOIP URI, it will jump to priority 30. If it is a normal telephone number it will jump to priority 40. If it is a MAILTO URI, it will jump to priority 50. If there are no more ENUM entries, or if it is an unsupported type, it will jump to priority 60.\n"
+"If the priority is <=0, it will be ignored.\n";
 
-static char *app = "EnumLookup";
+static char *old_app = "EnumLookup";
 
-static char *synopsis = "Lookup number in ENUM";
+static char *old_synopsis = "Lookup number in ENUM";
 
-static char *descrip = 
+static char *old_descrip = 
 "  EnumLookup(exten):  Looks up an extension via ENUM and sets\n"
 "the variable 'ENUM'. For VoIP URIs this variable will \n"
 "look like 'TECHNOLOGY/URI' with the appropriate technology.\n"
@@ -45,6 +78,7 @@
 "If the lookup was *not* successful and there exists a priority n + 101,\n"
 "then that priority will be taken next.\n" ;
 
+
 #define ENUM_CONFIG "enum.conf"
 
 static char h323driver[80] = "";
@@ -80,6 +114,7 @@
 				c += 4;
 			snprintf(tmp, sizeof(tmp), "SIP/%s", c);
 			pbx_builtin_setvar_helper(chan, "ENUM", tmp);
+			ast_log(LOG_NOTICE, "sip: ENUM set to \"%s\"\n", tmp);
 		} else if (!strcasecmp(tech, "h323")) {
 			c = dest;
 			if (!strncmp(c, "h323:", 5))
@@ -140,6 +175,136 @@
 	return res;
 }
 
+/*--- enumlookupng_exec: */
+static int enumlookupng_exec(struct ast_channel *chan, void *data)
+{
+	int res=0;
+	char tech[80];
+	char dest[80];
+	char tmp[256];
+	char *c,*t;
+	char number[256];
+	int  prio_tel;
+	int  prio_voip;
+	int  prio_fail;
+	int  prio_mailto;
+
+	struct localuser *u;
+
+
+	if (!data || ast_strlen_zero(data)) {
+		ast_log(LOG_WARNING, "EnumLookupNG: missing parameters\n");
+		return 1;
+	}
+
+	{
+		char *c;
+		int pcount;
+
+		strncpy(tmp,data,sizeof(tmp));
+		while ((c=strchr(tmp,'|'))!=NULL) {
+			*c=' ';
+		}
+		if ((pcount=sscanf(tmp,"%s %d %d %d %d",number,&prio_voip,&prio_tel,&prio_mailto,&prio_fail))!=5) {
+			ast_log(LOG_WARNING, "EnumLookupNG: missing parameters: %s\n",tmp);
+			return 1;
+		}
+	}
+	if (prio_voip>0 && !ast_exists_extension(chan, chan->context, chan->exten, prio_voip, chan->cid.cid_num))
+		ast_log(LOG_WARNING, "EnumLookupNG: missing or invalid VOIP priority (%d)\n",prio_voip);
+	if (prio_tel>0 && !ast_exists_extension(chan, chan->context, chan->exten, prio_tel, chan->cid.cid_num))
+		ast_log(LOG_WARNING, "EnumLookupNG: missing or invalid TEL priority (%d)\n",prio_tel);
+	if (prio_fail>0 && !ast_exists_extension(chan, chan->context, chan->exten, prio_fail, chan->cid.cid_num))
+		ast_log(LOG_WARNING, "EnumLookupNG: missing or invalid FAIL priority (%d)\n",prio_fail);
+	if (prio_mailto>0 && !ast_exists_extension(chan, chan->context, chan->exten, prio_mailto, chan->cid.cid_num))
+		ast_log(LOG_WARNING, "EnumLookupNG: missing or invalid MAILTO priority (%d)\n",prio_mailto);
+
+	LOCAL_USER_ADD(u);
+	if (!res) {
+		res = ast_get_enum(chan, number, dest, sizeof(dest), tech, sizeof(tech));
+		printf("ENUM got '%d'\n", res);
+	}
+	LOCAL_USER_REMOVE(u);
+	/* Parse it out */
+	if (res > 0) {
+		if (!strcasecmp(tech, "SIP")) {
+			c = dest;
+			if (!strncmp(c, "sip:", 4))
+				c += 4;
+			snprintf(tmp, sizeof(tmp), "SIP/%s", c);
+			pbx_builtin_setvar_helper(chan, "ENUM", tmp);
+			if (prio_voip>0)
+				chan->priority = prio_voip-1;
+		} else if (!strcasecmp(tech, "h323")) {
+			c = dest;
+			if (!strncmp(c, "h323:", 5))
+				c += 5;
+			snprintf(tmp, sizeof(tmp), "%s/%s", h323driver, c);
+/* do a s!;.*!! on the H323 URI */
+			t = strchr(c,';');
+			if (t) 
+				*t = 0;
+			pbx_builtin_setvar_helper(chan, "ENUM", tmp);
+			if (prio_voip>0)
+				chan->priority = prio_voip-1;
+		} else if (!strcasecmp(tech, "iax")) {
+			c = dest;
+			if (!strncmp(c, "iax:", 4))
+				c += 4;
+			snprintf(tmp, sizeof(tmp), "IAX/%s", c);
+			pbx_builtin_setvar_helper(chan, "ENUM", tmp);
+			if (prio_voip>0)
+				chan->priority = prio_voip-1;
+		} else if (!strcasecmp(tech, "iax2")) {
+			c = dest;
+			if (!strncmp(c, "iax2:", 5))
+				c += 5;
+			snprintf(tmp, sizeof(tmp), "IAX2/%s", c);
+			pbx_builtin_setvar_helper(chan, "ENUM", tmp);
+			if (prio_voip>0)
+				chan->priority = prio_voip-1;
+		} else if (!strcasecmp(tech, "mailto")) {
+			c = dest;
+			if (!strncmp(c, "mailto:", 7))
+				c += 7;
+			pbx_builtin_setvar_helper(chan, "ENUM", c);
+			if (prio_mailto>0)
+				chan->priority = prio_mailto-1;
+		} else if (!strcasecmp(tech, "tel")) {
+			c = dest;
+			if (!strncmp(c, "tel:", 4))
+				c += 4;
+
+			if (c[0] != '+') {
+				ast_log(LOG_NOTICE, "tel: uri must start with a \"+\" (got '%s')\n", c);
+				res = 0;
+			} else {
+/* now copy over the number, skipping all non-digits and stop at ; or NULL */
+				t = tmp;	
+				while( *c && (*c != ';') && (t - tmp < (sizeof(tmp) - 1))) {
+					if (isdigit(*c))
+						*t++ = *c;
+					c++;
+				}
+				*t = 0;
+				pbx_builtin_setvar_helper(chan, "ENUM", tmp);
+				if (prio_tel>0)
+					chan->priority = prio_tel-1;
+			}
+		} else if (!ast_strlen_zero(tech)) {
+			ast_log(LOG_NOTICE, "Don't know how to handle technology '%s'\n", tech);
+			chan->priority = prio_fail-1;
+			res = 0;
+		}
+	}
+	if (!res) {
+		/* Look for a "busy" place */
+		chan->priority = prio_fail-1;
+	} else if (res > 0)
+		res = 0;
+	return res;
+}
+
 static int load_config(void)
 {
 	struct ast_config *cfg;
@@ -162,14 +327,22 @@
 
 int unload_module(void)
 {
+	int res;
+
 	STANDARD_HANGUP_LOCALUSERS;
-	return ast_unregister_application(app);
+	res=ast_unregister_application(old_app);
+	res|=ast_unregister_application(new_app);
+	return res;
 }
 
 int load_module(void)
 {
 	int res;
-	res = ast_register_application(app, enumlookup_exec, synopsis, descrip);
+
+	/* Register EnumLookup */
+	res = ast_register_application(old_app,enumlookup_exec,old_synopsis,old_descrip);
+	/* Register EnumLookupNG */
+	res|=ast_register_application(new_app,enumlookupng_exec,new_synopsis,new_descrip);
 	if (res)
 		return(res);
 	if ((res=load_config())) {
